#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""test_hello_pure
----------------------------------

Tries to build and test the `hello-pure` sample project.
"""

import glob
import tarfile

from skbuild.constants import SKBUILD_DIR
from skbuild.utils import push_dir

from zipfile import ZipFile

from . import check_wheel_content, project_setup_py_test


@project_setup_py_test("hello-pure", ["build"], disable_languages_test=True)
def test_hello_pure_builds(capsys):
    out, _ = capsys.readouterr()
    assert "skipping skbuild (no CMakeLists.txt found)" in out


# @project_setup_py_test("hello-pure", ["test"])
# def test_hello_cython_works():
#     pass


@project_setup_py_test("hello-pure", ["sdist"], disable_languages_test=True)
def test_hello_pure_sdist():
    sdists_tar = glob.glob('dist/*.tar.gz')
    sdists_zip = glob.glob('dist/*.zip')
    assert sdists_tar or sdists_zip

    member_list = None
    expected_content = None
    if sdists_tar:
        expected_content = [
            'hello-pure-1.2.3',
            'hello-pure-1.2.3/hello',
            'hello-pure-1.2.3/hello/__init__.py',
            'hello-pure-1.2.3/setup.py',
            'hello-pure-1.2.3/PKG-INFO'
        ]
        member_list = tarfile.open('dist/hello-pure-1.2.3.tar.gz').getnames()

    elif sdists_zip:
        expected_content = [
            'hello-pure-1.2.3/hello/__init__.py',
            'hello-pure-1.2.3/setup.py',
            'hello-pure-1.2.3/PKG-INFO'
        ]
        member_list = ZipFile('dist/hello-pure-1.2.3.zip').namelist()

    assert expected_content and member_list
    assert sorted(expected_content) == sorted(member_list)


@project_setup_py_test("hello-pure", ["bdist_wheel"], disable_languages_test=True)
def test_hello_pure_wheel():
    expected_content = [
        'hello/__init__.py'
    ]

    expected_distribution_name = 'hello_pure-1.2.3'

    whls = glob.glob('dist/*.whl')
    assert len(whls) == 1
    check_wheel_content(whls[0], expected_distribution_name, expected_content, pure=True)


def test_hello_clean(capfd):
    with push_dir():

        @project_setup_py_test("hello-pure", ["build"], disable_languages_test=True)
        def run_build():
            pass

        tmp_dir = run_build()[0]

        assert tmp_dir.join(SKBUILD_DIR).exists()

        @project_setup_py_test("hello-pure", ["clean"], tmp_dir=tmp_dir, disable_languages_test=True)
        def run_clean():
            pass

        run_clean()

        assert not tmp_dir.join(SKBUILD_DIR).exists()

        out = capfd.readouterr()[0]
        assert 'Build files have been written to' not in out



"""This module defines objects useful to discover which CMake generator is
supported on the current platform."""

from __future__ import print_function

import os
import shutil
import subprocess
import textwrap

from ..constants import CMAKE_DEFAULT_EXECUTABLE
from ..exceptions import SKBuildGeneratorNotFoundError
from ..utils import push_dir

test_folder = "_cmake_test_compile"


class CMakePlatform(object):
    """This class encapsulates the logic allowing to get the identifier of a
    working CMake generator.

    Derived class should at least set :attr:`default_generators`.
    """
    def __init__(self):
        self._default_generators = list()

    @property
    def default_generators(self):
        """List of generators considered by :func:`get_best_generator()`."""
        return self._default_generators

    @default_generators.setter
    def default_generators(self, generators):
        self._default_generators = generators

    @property
    def generator_installation_help(self):
        """Return message guiding the user for installing a valid toolchain."""
        raise NotImplementedError  # pragma: no cover

    @staticmethod
    def write_test_cmakelist(languages):
        """Write a minimal ``CMakeLists.txt`` useful to check if the
        requested ``languages`` are supported."""
        if not os.path.exists(test_folder):
            os.makedirs(test_folder)
        with open("{:s}/{:s}".format(test_folder, "CMakeLists.txt"), "w") as f:
            f.write("cmake_minimum_required(VERSION 2.8)\n")
            f.write("PROJECT(compiler_test NONE)\n")
            for language in languages:
                f.write("ENABLE_LANGUAGE({:s})\n".format(language))

    @staticmethod
    def cleanup_test():
        """Delete test project directory."""
        if os.path.exists(test_folder):
            shutil.rmtree(test_folder)

    def get_generator(self, generator_name):
        """Loop over generators and return the first that matches the given
        name.
        """
        for default_generator in self.default_generators:
            if default_generator.name == generator_name:
                return default_generator

        return CMakeGenerator(generator_name)

    # TODO: this method name is not great.  Does anyone have a better idea for
    # renaming it?
    def get_best_generator(
            self, generator_name=None, skip_generator_test=False,
            languages=("CXX", "C"), cleanup=True,
            cmake_executable=CMAKE_DEFAULT_EXECUTABLE, cmake_args=()):
        """Loop over generators to find one that works by configuring
        and compiling a test project.

        :param generator_name: If provided, uses only provided generator, \
        instead of trying :attr:`default_generators`.
        :type generator_name: string or None

        :param skip_generator_test: If set to True and if a generator name is \
        specified, the generator test is skipped. If no generator_name is specified \
        and the option is set to True, the first available generator is used.
        :type skip_generator_test: bool

        :param languages: The languages you'll need for your project, in terms \
        that CMake recognizes.
        :type languages: tuple

        :param cleanup: If True, cleans up temporary folder used to test \
        generators. Set to False for debugging to see CMake's output files.
        :type cleanup: bool

        :param cmake_executable: Path to CMake executable used to configure \
        and build the test project used to evaluate if a generator is working.
        :type cmake_executable: string

        :param cmake_args: List of CMake arguments to use when configuring \
        the test project. Only arguments starting with ``-DCMAKE_`` are \
        used.
        :type cmake_args: tuple

        :return: CMake Generator object
        :rtype: :class:`CMakeGenerator` or None

        :raises skbuild.exceptions.SKBuildGeneratorNotFoundError:
        """

        candidate_generators = []

        if generator_name is None:
            candidate_generators = self.default_generators
        else:
            # Lookup CMakeGenerator by name. Doing this allow to get a
            # generator object with its ``env`` property appropriately
            # initialized.
            for default_generator in self.default_generators:
                if default_generator.name == generator_name:
                    candidate_generators = [default_generator]
                    break
            if not candidate_generators:
                candidate_generators = [CMakeGenerator(generator_name)]

        self.write_test_cmakelist(languages)

        if skip_generator_test:
            working_generator = candidate_generators[0]
        else:
            working_generator = self.compile_test_cmakelist(
                cmake_executable, candidate_generators, cmake_args)

        if working_generator is None:
            raise SKBuildGeneratorNotFoundError(textwrap.dedent(
                """
                {line}
                scikit-build could not get a working generator for your system. Aborting build.

                {installation_help}

                {line}
                """).strip().format(  # noqa: E501
                    line="*"*80,
                    installation_help=self.generator_installation_help)
            )

        if cleanup:
            CMakePlatform.cleanup_test()

        return working_generator

    @staticmethod
    @push_dir(directory=test_folder)
    def compile_test_cmakelist(
            cmake_exe_path, candidate_generators, cmake_args=()):
        """Attempt to configure the test project with
        each :class:`CMakeGenerator` from ``candidate_generators``.

        Only cmake arguments starting with ``-DCMAKE_`` are used to configure
        the test project.

        The function returns the first generator allowing to successfully
        configure the test project using ``cmake_exe_path``."""
        # working generator is the first generator we find that works.
        working_generator = None

        # Include only -DCMAKE_* arguments
        cmake_args = [arg for arg in cmake_args if arg.startswith("-DCMAKE_")]

        # Do not complain about unused CMake arguments
        cmake_args.insert(0, "--no-warn-unused-cli")

        cmake_args_as_str = " ".join(
            ['"{:s}"'.format(arg) for arg in cmake_args])

        def _generator_discovery_status_msg(_generator, suffix=""):
            outer = "-" * 80
            inner = ["-" * ((idx * 5) - 3) for idx in range(1, 8)]
            print(outer if suffix == "" else "\n".join(inner))
            print("-- Trying \"%s\" generator%s" % (_generator.name, suffix))
            print(outer if suffix != "" else "\n".join(inner[::-1]))

        for generator in candidate_generators:
            print("\n")
            _generator_discovery_status_msg(generator)

            # clear the cache for each attempted generator type
            if os.path.isdir('build'):
                shutil.rmtree('build')

            with push_dir('build', make_directory=True):
                # call cmake to see if the compiler specified by this
                # generator works for the specified languages
                cmake_execution_string = '{:s} ../ -G "{:s}" {:s}'.format(
                    cmake_exe_path, generator.name, cmake_args_as_str)
                status = subprocess.call(
                    cmake_execution_string, shell=True, env=generator.env)

            _generator_discovery_status_msg(
                generator, " - %s" % ("success" if status == 0 else "failure"))
            print("")

            # cmake succeeded, this generator should work
            if status == 0:
                # we have a working generator, don't bother looking for more
                working_generator = generator
                break

        return working_generator


class CMakeGenerator(object):
    """Represents a CMake generator.

    .. automethod:: __init__
    """

    def __init__(self, name, env=None):
        """Instantiate a generator object with the given ``name``.

        By default, ``os.environ`` is associated with the generator. Dictionary
        passed as ``env`` parameter will be merged with ``os.environ``. If an
        environment variable is set in both ``os.environ`` and ``env``, the
        variable in ``env`` is used.
        """
        self._generator_name = name
        self.env = dict(
            list(os.environ.items()) + list(env.items() if env else []))

    @property
    def name(self):
        """Name of CMake generator."""
        return self._generator_name



"""
This module defines constants commonly used in scikit-build.
"""

import os
import sys

from distutils.util import get_platform

SKBUILD_DIR = os.path.join(
    "_skbuild",
    "{}-{}".format(get_platform(), '.'.join(map(str, sys.version_info[:2]))),
)
"""Top-level directory where setuptools and CMake directories are generated."""

CMAKE_DEFAULT_EXECUTABLE = "cmake"
"""Default path to CMake executable."""

CMAKE_BUILD_DIR = os.path.join(SKBUILD_DIR, "cmake-build")
"""CMake build directory."""

CMAKE_INSTALL_DIR = os.path.join(SKBUILD_DIR, "cmake-install")
"""CMake install directory."""

CMAKE_SPEC_FILE = os.path.join(CMAKE_BUILD_DIR, "CMakeSpec.json")
"""CMake specification file storing CMake version, CMake configuration arguments and
environment variables ``PYTHONNOUSERSITE`` and ``PYTHONPATH``.
"""

SETUPTOOLS_INSTALL_DIR = os.path.join(SKBUILD_DIR, "setuptools")
"""Setuptools install directory."""



"""This module defines custom implementation of ``egg_info`` setuptools
command."""

import os
import os.path

from setuptools.command.egg_info import egg_info as _egg_info

from . import set_build_base_mixin
from ..constants import CMAKE_INSTALL_DIR
from ..utils import new_style, to_unix_path


class egg_info(set_build_base_mixin, new_style(_egg_info)):
    """Custom implementation of ``egg_info`` setuptools command."""

    def finalize_options(self, *args, **kwargs):
        # pylint:disable=access-member-before-definition
        if self.egg_base is None:
            if self.distribution.package_dir is not None and len(self.distribution.package_dir) == 1:
                # Recover directory specified in setup() function
                # using `package_dir={'':<egg_base>}`
                # This is required to successfully update the python path when
                # running the test command.
                package_name = list(self.distribution.package_dir.keys())[0]
                egg_base = to_unix_path(list(self.distribution.package_dir.values())[0])
                cmake_install_dir = to_unix_path(CMAKE_INSTALL_DIR)
                if egg_base.startswith(cmake_install_dir):
                    egg_base = egg_base[len(cmake_install_dir) + 1:]
                if package_name and egg_base.endswith(package_name):
                    egg_base = egg_base[:-len(package_name) - 1]
                if egg_base == "":
                    egg_base = "."
                # pylint:disable=attribute-defined-outside-init
                self.egg_base = egg_base
        else:
            script_path = os.path.abspath(self.distribution.script_name)
            script_dir = os.path.dirname(script_path)
            # pylint:disable=attribute-defined-outside-init
            self.egg_base = os.path.join(script_dir, self.egg_base)

        super(egg_info, self).finalize_options(*args, **kwargs)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""test_manifest_in
----------------------------------

Tries to build and test the `manifest-in` sample project.
"""

import glob
import tarfile
import wheel

from pkg_resources import parse_version
from zipfile import ZipFile

from . import project_setup_py_test


@project_setup_py_test("manifest-in", ["sdist"], disable_languages_test=True)
def test_manifest_in_sdist():
    sdists_tar = glob.glob('dist/*.tar.gz')
    sdists_zip = glob.glob('dist/*.zip')
    assert sdists_tar or sdists_zip

    member_list = None
    expected_content = None
    if sdists_tar:
        expected_content = [
            'manifest-in-1.2.3',
            'manifest-in-1.2.3/hello',
            'manifest-in-1.2.3/hello/__init__.py',
            'manifest-in-1.2.3/setup.py',
            'manifest-in-1.2.3/PKG-INFO'
        ]
        member_list = tarfile.open('dist/manifest-in-1.2.3.tar.gz').getnames()

    elif sdists_zip:
        expected_content = [
            'manifest-in-1.2.3/hello/__init__.py',
            'manifest-in-1.2.3/setup.py',
            'manifest-in-1.2.3/PKG-INFO'
        ]
        member_list = ZipFile('dist/manifest-in-1.2.3.zip').namelist()

    assert expected_content and member_list
    assert sorted(expected_content) == sorted(member_list)


@project_setup_py_test("manifest-in", ["bdist_wheel"], disable_languages_test=True)
def test_manifest_in_wheel():
    whls = glob.glob('dist/*.whl')
    assert len(whls) == 1
    assert whls[0].endswith('-none-any.whl')

    expected_content = [
        'manifest_in-1.2.3.dist-info/top_level.txt',
        'manifest_in-1.2.3.dist-info/WHEEL',
        'manifest_in-1.2.3.dist-info/RECORD',
        'manifest_in-1.2.3.dist-info/METADATA',
        'hello/__init__.py'
    ]

    if parse_version(wheel.__version__) < parse_version('0.31.0'):
        expected_content += [
            'manifest_in-1.2.3.dist-info/DESCRIPTION.rst',
            'manifest_in-1.2.3.dist-info/metadata.json'
        ]

    member_list = ZipFile(whls[0]).namelist()
    assert sorted(expected_content) == sorted(member_list)



"""This module defines custom implementation of ``clean`` setuptools command."""

import os

try:
    from setuptools.command.clean import clean as _clean
except ImportError:
    from distutils.command.clean import clean as _clean

from distutils import log

from shutil import rmtree

from . import set_build_base_mixin
from ..constants import (CMAKE_BUILD_DIR,
                         CMAKE_INSTALL_DIR,
                         SKBUILD_DIR)
from ..utils import new_style


class clean(set_build_base_mixin, new_style(_clean)):
    """Custom implementation of ``clean`` setuptools command."""
    def run(self):
        """After calling the super class implementation, this function removes
        the directories specific to scikit-build."""
        super(clean, self).run()
        for dir_ in (CMAKE_INSTALL_DIR,
                     CMAKE_BUILD_DIR,
                     SKBUILD_DIR):
            if os.path.exists(dir_):
                log.info("removing '%s'", dir_)
            if not self.dry_run and os.path.exists(dir_):
                rmtree(dir_)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""test_utils
------------------------

Tests for utils functions.
"""

import os
import pytest

from skbuild.utils import (ContextDecorator, mkdir_p,
                           PythonModuleFinder, push_dir,
                           to_platform_path, to_unix_path)

from . import (push_env, SAMPLES_DIR)

saved_cwd = os.getcwd()


def setup_module():
    """ setup any state specific to the execution of the given module."""
    os.chdir(os.path.dirname(__file__))


def teardown_module():
    """ teardown any state that was previously setup with a setup_module
    method.
    """
    os.chdir(saved_cwd)


def test_context_decorator():
    with ContextDecorator(foo=42) as context:
        assert hasattr(context, 'foo')
        assert context.foo == 42


def test_push_dir(tmpdir):
    old_cwd = os.getcwd()
    try:
        level1 = tmpdir.mkdir("level1")
        level2 = level1.mkdir("level2")

        os.chdir(str(level2))
        assert os.path.split(os.getcwd())[-1] == 'level2'

        # No directory
        with push_dir():
            os.chdir(os.path.join(os.getcwd(), '..'))
            assert os.path.split(os.getcwd())[-1] == 'level1'
        assert os.path.split(os.getcwd())[-1] == 'level2'

        # With existing directory
        with push_dir(directory=os.path.join(os.getcwd(), '..')):
            assert os.path.split(os.getcwd())[-1] == 'level1'
        assert os.path.split(os.getcwd())[-1] == 'level2'

        foo_directory = os.path.join(str(tmpdir), 'foo')

        # With non existing directory
        failed = False
        try:
            with push_dir(directory=foo_directory):
                pass
        except OSError:
            failed = True
        assert failed
        assert not os.path.isdir(foo_directory)

        # With make_directory option
        with push_dir(directory=foo_directory, make_directory=True):
            assert os.getcwd() == foo_directory
        assert os.path.split(os.getcwd())[-1] == 'level2'
        assert os.path.isdir(foo_directory)
    finally:
        os.chdir(old_cwd)


def test_push_dir_decorator(tmpdir):
    old_cwd = os.getcwd()
    try:
        level1 = tmpdir.mkdir("level1")
        level2 = level1.mkdir("level2")
        os.chdir(str(level2))
        assert os.path.split(os.getcwd())[-1] == 'level2'

        # No directory
        @push_dir()
        def test_default():
            os.chdir(os.path.join(os.getcwd(), '..'))
            assert os.path.split(os.getcwd())[-1] == 'level1'

        test_default()
        assert os.path.split(os.getcwd())[-1] == 'level2'

        # With existing directory
        @push_dir(directory=os.path.join(os.getcwd(), '..'))
        def test():
            assert os.path.split(os.getcwd())[-1] == 'level1'

        test()
        assert os.path.split(os.getcwd())[-1] == 'level2'

        foo_directory = os.path.join(str(tmpdir), 'foo')

        # With non existing directory
        failed = False
        try:
            @push_dir(directory=foo_directory)
            def test():
                pass

            test()
        except OSError:
            failed = True
        assert failed
        assert not os.path.isdir(foo_directory)

        # With make_directory option
        @push_dir(directory=foo_directory, make_directory=True)
        def test():
            assert os.getcwd() == foo_directory

        test()
        assert os.path.split(os.getcwd())[-1] == 'level2'
        assert os.path.isdir(foo_directory)
    finally:
        os.chdir(old_cwd)


def test_mkdir_p(tmpdir):
    tmp_dir = str(tmpdir)
    assert os.path.isdir(tmp_dir)

    foo_bar_dir = os.path.join(tmp_dir, 'foo', 'bar')

    mkdir_p(foo_bar_dir)
    assert os.path.isdir(foo_bar_dir)

    # Make sure calling function twice does not raise an exception
    mkdir_p(foo_bar_dir)
    assert os.path.isdir(foo_bar_dir)


def test_push_env():
    assert 'SKBUILD_NEW_VAR' not in os.environ

    os.environ['SKBUILD_ANOTHER_VAR'] = 'abcd'
    assert 'SKBUILD_ANOTHER_VAR' in os.environ

    saved_env = dict(os.environ)

    # Setting and un-setting variables can be done simultaneously
    with push_env(SKBUILD_NEW_VAR='1234', SKBUILD_ANOTHER_VAR=None):
        assert 'SKBUILD_NEW_VAR' in os.environ
        assert 'SKBUILD_ANOTHER_VAR' not in os.environ
        assert os.getenv('SKBUILD_NEW_VAR') == '1234'

    assert 'SKBUILD_NEW_VAR' not in os.environ
    assert 'SKBUILD_ANOTHER_VAR' in os.environ
    assert saved_env == os.environ

    # Trying to unset an unknown variable should be a no-op
    with push_env(SKBUILD_NOT_SET=None):
        assert saved_env == os.environ
    assert saved_env == os.environ

    # Calling without argument should be a no-op
    with push_env():
        assert saved_env == os.environ
    assert saved_env == os.environ


def test_python_module_finder():
    modules = PythonModuleFinder(['bonjour', 'hello'], {}, []).find_all_modules(
        os.path.join(SAMPLES_DIR, 'hello-cpp')
    )
    assert sorted(modules) == sorted([
        ('bonjour', '__init__', to_platform_path('bonjour/__init__.py')),
        ('hello', '__init__', to_platform_path('hello/__init__.py')),
        ('hello', '__main__', to_platform_path('hello/__main__.py'))])


@pytest.mark.parametrize(
    "input_path, expected_path", (
        (None, None),
        ('', ''),
        ('/bar/foo/baz', '{s}bar{s}foo{s}baz'.format(s=os.sep)),
        ('C:\\bar\\foo\\baz', 'C:{s}bar{s}foo{s}baz'.format(s=os.sep)),
        ('C:\\bar/foo\\baz/', 'C:{s}bar{s}foo{s}baz{s}'.format(s=os.sep)),
    )
)
def test_to_platform_path(input_path, expected_path):
    assert to_platform_path(input_path) == expected_path


@pytest.mark.parametrize(
    "input_path, expected_path", (
        (None, None),
        ('', ''),
        ('/bar/foo/baz', '/bar/foo/baz'),
        ('C:\\bar\\foo\\baz', 'C:/bar/foo/baz'),
        ('C:\\bar/foo\\baz/', 'C:/bar/foo/baz/'),
    )
)
def test_to_unix_path(input_path, expected_path):
    assert to_unix_path(input_path) == expected_path



import pytest
import sys
import textwrap

from . import _tmpdir, execute_setup_py


@pytest.mark.parametrize("cli_setup_args,keyword_cmake_args,cli_cmake_args,expected_cmake_osx_deployment_target", [
    # default plat_name is 'macosx-10.6-x86_64'
    (
            # cli_setup_args
            [],
            # keyword_cmake_args
            [],
            # cli_cmake_args
            [],
            # expected_cmake_osx_deployment_target
            "10.6"
    ),
    (
            # cli_setup_args
            ['--plat-name', 'macosx-10.6-x86_64'],
            # keyword_cmake_args
            [],
            # cli_cmake_args
            [],
            # expected_cmake_osx_deployment_target
            "10.6"
    ),
    (
            # cli_setup_args
            ['--plat-name', 'macosx-10.7-x86_64'],
            # keyword_cmake_args
            [],
            # cli_cmake_args
            [],
            # expected_cmake_osx_deployment_target
            "10.7"
    ),
    (
            # cli_setup_args
            [],
            # keyword_cmake_args
            ['-DCMAKE_OSX_DEPLOYMENT_TARGET:STRING=10.7'],
            # cli_cmake_args
            [],
            # expected_cmake_osx_deployment_target
            "10.7"
    ),
    (
            # cli_setup_args
            ['--plat-name', 'macosx-10.12-x86_64'],
            # keyword_cmake_args
            ['-DCMAKE_OSX_DEPLOYMENT_TARGET:STRING=10.7'],
            # cli_cmake_args
            [],
            # expected_cmake_osx_deployment_target
            "10.7"
    ),
    (
            # cli_setup_args
            [],
            # keyword_cmake_args
            ['-DCMAKE_OSX_DEPLOYMENT_TARGET:STRING=10.7'],
            # cli_cmake_args
            ['-DCMAKE_OSX_DEPLOYMENT_TARGET:STRING=10.8'],
            # expected_cmake_osx_deployment_target
            "10.8"
    ),
    (
            # cli_setup_args
            ['--plat-name', 'macosx-10.12-x86_64'],
            # keyword_cmake_args
            ['-DCMAKE_OSX_DEPLOYMENT_TARGET:STRING=10.7'],
            # cli_cmake_args
            ['-DCMAKE_OSX_DEPLOYMENT_TARGET:STRING=10.8'],
            # expected_cmake_osx_deployment_target
            "10.8"
    ),
])
def test_cmake_args_keyword_osx_default(
        cli_setup_args, keyword_cmake_args, cli_cmake_args, expected_cmake_osx_deployment_target, mocker):

    tmp_dir = _tmpdir('cmake_args_keyword_osx_default')

    tmp_dir.join('setup.py').write(textwrap.dedent(
        """
        from skbuild import setup
        setup(
            name="test_cmake_args_keyword_osx_default",
            version="1.2.3",
            description="A minimal example package",
            author="The scikit-build team",
            license="MIT",
            cmake_args=[{cmake_args}]
        )
        """.format(cmake_args=",".join(["'%s'" % arg for arg in keyword_cmake_args]))
    ))
    tmp_dir.join('CMakeLists.txt').write(textwrap.dedent(
        """
        message(FATAL_ERROR "This error message should not be displayed")
        """
    ))

    mock_configure = mocker.patch('skbuild.cmaker.CMaker.configure', side_effect=RuntimeError("exit skbuild"))

    try:
        # allow to run the test on any platform
        saved_platform = sys.platform
        sys.platform = "darwin"

        with pytest.raises(RuntimeError, match="exit skbuild"):
            with execute_setup_py(tmp_dir, ['build'] + cli_setup_args + ['--'] + cli_cmake_args):
                pass
    finally:
        sys.platform = saved_platform

    assert mock_configure.call_count == 1

    current_cmake_args = mock_configure.call_args[0][0]

    # Since additional cmake argument are appended, it is not possible to simply
    # compare lists.
    found_cmake_osx_deployment_target = False
    for cmake_arg in reversed(current_cmake_args):
        if cmake_arg.startswith("-DCMAKE_OSX_DEPLOYMENT_TARGET:STRING="):
            if cmake_arg.endswith(expected_cmake_osx_deployment_target):
                found_cmake_osx_deployment_target = True
            break

    assert found_cmake_osx_deployment_target, \
        textwrap.dedent("""
                    Argument -DCMAKE_OSX_DEPLOYMENT_TARGET:STRING=%s is NOT found near the end of
                    current list of arguments:
                      keyword_cmake_args  : %s
                      cli_cmake_args    : %s
                      current_cmake_args: %s
                    """ % (expected_cmake_osx_deployment_target,
                           keyword_cmake_args, cli_cmake_args, current_cmake_args))



"""This module defines object specific to Windows platform."""

from __future__ import print_function

import sys
import platform
import textwrap

from .abstract import CMakeGenerator

from . import abstract


class WindowsPlatform(abstract.CMakePlatform):
    """Windows implementation of :class:`.abstract.CMakePlatform`."""

    def __init__(self):
        super(WindowsPlatform, self).__init__()
        version = sys.version_info
        self._vs_help = ""
        vs_help_template = textwrap.dedent(
            """
            Building windows wheels for Python {pyver} requires Microsoft Visual Studio %s.
            Get it with "%s":

              %s
            """  # noqa: E501
        ).strip().format(pyver="%s.%s" % sys.version_info[:2])

        # For Python 2.7 to Python 3.2: VS2008
        if (
            (version.major == 2 and version.minor >= 7) or
            (version.major == 3 and version.minor <= 2)
        ):
            official_vs_year = "2008"
            self._vs_help = vs_help_template % (
                official_vs_year,
                "Microsoft Visual C++ Compiler for Python 2.7",
                "http://aka.ms/vcpython27"
            )

        # For Python 3.3 to Python 3.4: VS2010
        elif (
            version.major == 3 and (
                version.minor >= 3 and
                version.minor <= 4
            )
        ):
            official_vs_year = "2010"
            self._vs_help = vs_help_template % (
                official_vs_year,
                "Windows SDK for Windows 7 and .NET 4.0",
                "https://www.microsoft.com/download/details.aspx?id=8279"
            )
            #

        # For Python 3.5 and above: VS2015
        elif version.major == 3 and version.minor >= 5:
            official_vs_year = "2015"
            self._vs_help = vs_help_template % (
                official_vs_year,
                "Microsoft Visual C++ Build Tools",
                "http://landinghub.visualstudio.com/visual-cpp-build-tools"
            )
            self._vs_help += "\n\n" + textwrap.dedent(
                """
                Or with "Visual Studio 2015":

                  https://visualstudio.com/
                """
            ).strip()

        else:
            raise RuntimeError("Only Python >= 2.7 is supported on Windows.")

        assert official_vs_year is not None

        supported_vs_years = [official_vs_year]

        for vs_year in supported_vs_years:
            self.default_generators.extend([
                CMakeVisualStudioCommandLineGenerator("Ninja",
                                                      vs_year),
                CMakeVisualStudioIDEGenerator(vs_year),
                CMakeVisualStudioCommandLineGenerator(
                    "NMake Makefiles", vs_year),
                CMakeVisualStudioCommandLineGenerator(
                    "NMake Makefiles JOM", vs_year)
            ])

    @property
    def generator_installation_help(self):
        """Return message guiding the user for installing a valid toolchain."""
        return self._vs_help


VS_YEAR_TO_VERSION = {
    "2008": 9,
    "2010": 10,
    "2015": 14
}
"""Describes the version of `Visual Studio` supported by
:class:`CMakeVisualStudioIDEGenerator` and
:class:`CMakeVisualStudioCommandLineGenerator`.

The different version are identified by their year.
"""


class CMakeVisualStudioIDEGenerator(CMakeGenerator):
    """
    Represents a Visual Studio CMake generator.

    .. automethod:: __init__
    """
    def __init__(self, year):
        """Instantiate a generator object with its name set to the `Visual
        Studio` generator associated with the given ``year``
        (see :data:`VS_YEAR_TO_VERSION`) and the current platform (32-bit
        or 64-bit).
        """
        vs_version = VS_YEAR_TO_VERSION[year]
        vs_base = "Visual Studio %s %s" % (vs_version, year)
        # Python is Win64, build a Win64 module
        if platform.architecture()[0] == "64bit":
            vs_base += " Win64"
        super(CMakeVisualStudioIDEGenerator, self).__init__(vs_base)


# To avoid multiple slow calls to ``query_vcvarsall`` or ``_get_vc_env``, results
# of previous calls are cached.
__get_msvc_compiler_env_cache = dict()


def _get_msvc_compiler_env(vs_version):
    # pylint:disable=global-statement
    global __get_msvc_compiler_env_cache
    from setuptools import monkey
    monkey.patch_for_msvc_specialized_compiler()
    arch = "x86"
    if vs_version < 14:
        if platform.architecture()[0] == "64bit":
            arch = "amd64"
        # If any, return cached version
        cache_key = ",".join([str(vs_version), arch])
        if cache_key in __get_msvc_compiler_env_cache:
            return __get_msvc_compiler_env_cache[cache_key]
        try:
            import distutils.msvc9compiler
            cached_env = distutils.msvc9compiler.query_vcvarsall(vs_version, arch)
            __get_msvc_compiler_env_cache[cache_key] = cached_env
            return cached_env
        except ImportError:
            print("failed to import 'distutils.msvc9compiler'")
    else:
        if platform.architecture()[0] == "64bit":
            arch = "x86_amd64"
        # If any, return cached version
        cache_key = ",".join([str(vs_version), arch])
        if cache_key in __get_msvc_compiler_env_cache:
            return __get_msvc_compiler_env_cache[cache_key]
        try:
            import distutils._msvccompiler
            from distutils.errors import DistutilsPlatformError
            # pylint:disable=protected-access
            vc_env = distutils._msvccompiler._get_vc_env(arch)
            cached_env = {
                'PATH': vc_env.get('path', ''),
                'INCLUDE': vc_env.get('include', ''),
                'LIB': vc_env.get('lib', '')
            }
            __get_msvc_compiler_env_cache[cache_key] = cached_env
            return cached_env
        except ImportError:
            print("failed to import 'distutils._msvccompiler'")
        except DistutilsPlatformError:
            pass
    return {}


class CMakeVisualStudioCommandLineGenerator(CMakeGenerator):
    """
    Represents a command-line CMake generator initialized with a
    specific `Visual Studio` environment.

    .. automethod:: __init__
    """
    def __init__(self, name, year):
        """Instantiate CMake command-line generator.

        The generator ``name`` can be values like `Ninja`, `NMake Makefiles`
        or `NMake Makefiles JOM`.

        The ``year`` defines the `Visual Studio` environment associated
        with the generator. See :data:`VS_YEAR_TO_VERSION`.

        The platform (32-bit or 64-bit) is automatically selected based
        on the value of ``platform.architecture()[0]``.
        """
        vc_env = _get_msvc_compiler_env(VS_YEAR_TO_VERSION[year])
        env = {str(key.upper()): str(value) for key, value in vc_env.items()}
        super(CMakeVisualStudioCommandLineGenerator, self).__init__(name, env)



"""This module defines custom implementation of ``bdist_wheel`` setuptools
command."""

import sys

from wheel import archive as _wheel_archive
from wheel.bdist_wheel import bdist_wheel as _bdist_wheel

from . import set_build_base_mixin
from ..utils import distribution_hide_listing, new_style


class bdist_wheel(set_build_base_mixin, new_style(_bdist_wheel)):
    """Custom implementation of ``bdist_wheel`` setuptools command."""

    def run(self, *args, **kwargs):
        """Handle --hide-listing option."""

        old_make_wheelfile_inner = _wheel_archive.make_wheelfile_inner

        def _make_wheelfile_inner(base_name, base_dir='.'):
            with distribution_hide_listing(self.distribution) as hide_listing:
                if hide_listing:
                    zip_filename = base_name + ".whl"
                    print("creating '%s' and adding '%s' to it"
                          % (zip_filename, base_dir))
                old_make_wheelfile_inner(base_name, base_dir)

        _wheel_archive.make_wheelfile_inner = _make_wheelfile_inner

        try:
            super(bdist_wheel, self).run(*args, **kwargs)
        finally:
            _wheel_archive.make_wheelfile_inner = old_make_wheelfile_inner

    def finalize_options(self, *args, **kwargs):
        """Ensure MacOSX wheels include ``x86_64`` instead of ``intel``."""
        # pylint:disable=attribute-defined-outside-init,access-member-before-definition
        if sys.platform == 'darwin' and self.plat_name is None and self.distribution.has_ext_modules():
            # The following code is duplicated in setuptools_wrap
            # pylint:disable=access-member-before-definition
            self.plat_name = "macosx-10.6-x86_64"
        super(bdist_wheel, self).finalize_options(*args, **kwargs)

    def write_wheelfile(self, wheelfile_base, _=None):
        """Write ``skbuild <version>`` as a wheel generator.
        See `PEP-0427 <https://www.python.org/dev/peps/pep-0427/#file-contents>`_ for more details.
        """
        from .. import __version__ as skbuild_version
        generator = "skbuild %s" % skbuild_version
        super(bdist_wheel, self).write_wheelfile(wheelfile_base, generator)
